pragma solidity ^0.4.24;

contract sueldo {
    
    //Estrucutra de datos de los empleados
    struct perfilEmpleado {
        address direccion;//dirección de cuenta de usuario de Ethereum
        bytes16 nombre;//nombre del empleado
        bytes16 apellidos;//apellidos del empleado
        uint horas; //horas trabajadas que registra el empleado
        uint  aPagar;//sueldo que recibe por las horas trabajadas
        uint totalPagado;//total que le han pagado del total de veces contratado
    }

    //Precio hora de trabajo
    uint internal precioHora = 1000000000000000000;
    
    //@dev returns el sueldo total del trabajador en función de las horas trabajadas
    //@ arg horas trabajadas
    function sueldoEmpleado(uint _horas) internal view returns (uint aPagar){
        uint totalEther = (_horas*precioHora)/1000000000000000000;
        return totalEther;
    }
    //Evento paa disparar eventos en el UI 
    event fichar(
        bytes16 nombre,
        bytes16 apellidos,
        uint horas,
        uint aPagar,
        uint totalPagado,
        address direccion
    );
    
    //Variable para guardar la estruc. de datos de los empleados
    mapping (address => perfilEmpleado) internal listaEmpleados;
    //Variable de tipo array para guardar direcciones de Ethereum de los empleados
    address[] cuentaEmpleados;
    
    //@dev Establece los datos de un empleado
    //@arg _direccion, _nombre, _apellidos, _horas 
    /*****Para el ejemplo del TFG los datos se rellenan manualmente como si el propio empleado
    /*****fuese quien rellenase los campos. No obstante esta función podría estar alimentada automáticamente 
    /*****desde una base de datos del sistema de reconocimiento biométrico****/
    function setEmpleado(address _direccion, bytes16 _nombre, bytes16 _apellidos, uint _horas) public {
        
        var empleado = listaEmpleados[_direccion]; //Variable de tipo mapping
        
        //Establecemos el valor de cada variable de la estructura de datos de empleado
        empleado.direccion = _direccion;
        empleado.nombre = _nombre;
        empleado.apellidos = _apellidos;
        empleado.horas = _horas;
        empleado.aPagar = sueldoEmpleado(_horas);
        //Acumula el total de lo que el empleado ha recibido de la empresa
        empleado.totalPagado = totalPagadoAcumulado(empleado.direccion, empleado.aPagar, empleado.totalPagado); 
        //Variable que no se pasa desde el UI por lo que la declaramos dentro del setter
        uint aPagar = empleado.aPagar; 
        //Llamamos a la función para actualizar el balance total del contrato y pagar al empleado
        //Si lanza una excepción se revierte toda la transacción del empleado
        updateBalanceAndPay(aPagar, empleado.direccion);
        //guardamos la dirección de usuario en el array
        cuentaEmpleados.push(_direccion) -1;
        //declaramos la variable para poder pasarla al evento
        uint totalPagado = empleado.totalPagado;
        //Llamamos al evento
        emit fichar(_nombre, _apellidos, _horas, aPagar, totalPagado, _direccion);
   }
    
    //Esta función comprueba el balance total del contrato, si hay menos saldo de lo que hay que 
    //pagar, lanza una excepción y revierte la transacción del empleado. 
    //El gas de la transacción no se recupera
    function updateBalanceAndPay(uint _aPagar, address _direccion) internal {
        //Descontamos el sueldo del empleado del balance total del contrato
        uint balance  = address(this).balance;
        require (balance > _aPagar);//Si hay menos balance se revierte la transacción a partir de aquí
        uint subTotal = address(this).balance - _aPagar;
        uint totalPagar = (balance - subTotal)*1000000000000000000;
        _direccion.transfer(totalPagar);
        //transferimos en ether el sueldo la cuenta de usuario de Ethereum del empleado
    } 
    //Función que calcula y guarda el total de los saldos que ha cobrado un empleado concreto
    function totalPagadoAcumulado (address _direccion, uint aPagar, uint _totalPagado) internal returns (uint acumulado){
        uint x = listaEmpleados[_direccion].totalPagado;
        uint y = aPagar;
        return acumulado = x+y;
    }
   
   //Función que recupera lso datos de un empleado
   function getEmpleado(address _direccion)  view public returns (bytes16 nombre, bytes16 apellidos, uint horas, uint aPagar, uint totalPagado, address direccion) {
        return(listaEmpleados[_direccion].nombre, 
                listaEmpleados[_direccion].apellidos, 
                listaEmpleados[_direccion].horas, 
                listaEmpleados[_direccion].aPagar,
                listaEmpleados[_direccion].totalPagado,
                listaEmpleados[_direccion].direccion);
   }  
   //Función que deposita fondos en el contrato
   function deposit() payable public {
        // nothing to do!
    }
   //Función para recuperar el balance de fondos del contrato
    function getBalance() public view returns (uint256) {
        return address(this).balance;
    }
    //Función para recuperar las direcciones de los empleados guardadas
   function getDireccionesEmpleados() view  public returns (address[]) {
       return cuentaEmpleados;
   }
   //Función para saber el número total de empleados guardados
    function numeroEmpleados() view public returns (uint) {
       return cuentaEmpleados.length;
    }    
}